
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "antPropertiesPage.h"
#include "layLayoutView.h"

namespace ant
{

// -------------------------------------------------------------------------
//  PropertiesPage implementation

PropertiesPage::PropertiesPage (ant::Service *rulers, QWidget *parent)
  : lay::PropertiesPage (parent, rulers), mp_rulers (rulers), m_enable_cb_callback (true)
{
  mp_rulers->get_selection (m_selection);
  m_pos = m_selection.begin ();

  setupUi (this);

  mp_rulers->clear_highlights ();
}

PropertiesPage::~PropertiesPage ()
{
  mp_rulers->restore_highlights ();
}

void 
PropertiesPage::back ()
{
  m_pos = m_selection.end ();
}

void 
PropertiesPage::front ()
{
  m_pos = m_selection.begin ();
}

const ant::Object &
PropertiesPage::current () const
{
  const ant::Object *ruler = dynamic_cast <const ant::Object *> ((*m_pos)->ptr ());
  return *ruler;
}

bool 
PropertiesPage::at_begin () const
{
  return (m_pos == m_selection.begin ());
}

bool 
PropertiesPage::at_end () const
{
  return (m_pos == m_selection.end ());
}

void 
PropertiesPage::operator-- ()
{
  --m_pos;
}

void 
PropertiesPage::operator++ ()
{
  ++m_pos;
}

void
PropertiesPage::leave ()
{
  mp_rulers->clear_highlights ();
}

void 
PropertiesPage::update ()
{
  mp_rulers->highlight (std::distance (m_selection.begin (), m_pos));

  fmt_le->setText (current ().fmt ().c_str ());
  fmt_x_le->setText (current ().fmt_x ().c_str ());
  fmt_y_le->setText (current ().fmt_y ().c_str ());
  style_cb->setCurrentItem (current ().style ());
  outline_cb->setCurrentItem (current ().outline ());

  x1->setText (tl::micron_to_string (current ().p1 ().x ()).c_str ());
  x1->setCursorPosition (0);
  x2->setText (tl::micron_to_string (current ().p2 ().x ()).c_str ());
  x2->setCursorPosition (0);
  y1->setText (tl::micron_to_string (current ().p1 ().y ()).c_str ());
  y1->setCursorPosition (0);
  y2->setText (tl::micron_to_string (current ().p2 ().y ()).c_str ());
  y2->setCursorPosition (0);

  double sx = (current ().p2 ().x () - current ().p1 ().x ());
  double sy = (current ().p2 ().y () - current ().p1 ().y ());
  dx->setText (tl::micron_to_string (sx).c_str ());
  dx->setCursorPosition (0);
  dy->setText (tl::micron_to_string (sy).c_str ());
  dy->setCursorPosition (0);
  dd->setText (tl::micron_to_string (sqrt (sx * sx + sy * sy)).c_str ());
  dd->setCursorPosition (0);
}

bool 
PropertiesPage::readonly ()
{
  return false;
}

void 
PropertiesPage::apply ()
{
  double dx1, dy1, dx2, dy2;

  tl::from_string (x1->text ().ascii (), dx1);
  tl::from_string (x2->text ().ascii (), dx2);
  tl::from_string (y1->text ().ascii (), dy1);
  tl::from_string (y2->text ().ascii (), dy2);

  std::string fmt = fmt_le->text ().ascii ();
  std::string fmt_x = fmt_x_le->text ().ascii ();
  std::string fmt_y = fmt_y_le->text ().ascii ();
  Object::style_type style = Object::style_type (style_cb->currentItem ());
  Object::outline_type outline = Object::outline_type (outline_cb->currentItem ());

  ant::Object ruler (db::DPoint (dx1, dy1), db::DPoint (dx2, dy2), current ().id (), fmt_x, fmt_y, fmt, style, outline, current ().snap (), current ().angle_constraint ());

  mp_rulers->change_ruler (*m_pos, ruler);
}

}

