
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <string>
#include <map>

#include "dbGDS2Converter.h"
#include "dbGDS2TextReader.h"
#include "dbGDS2TextWriter.h"
#include "dbGDS2.h"
#include "dbStream.h"

#include "tlClassRegistry.h"

namespace db
{

// ---------------------------------------------------------------
//  GDS2 Text format implementation

class GDS2TextFormatDeclaration
  : public db::StreamFormatDeclaration
{
  virtual std::string format_name () const { return "GDS2Text"; }
  virtual std::string format_desc () const { return "GDS2 Text Stream Format"; }
  virtual std::string file_format () const { return "GDS2 Text files (*.txt *.TXT )"; }

  virtual bool detect (const char hdr[4]) const 
  {
    std::string sHeader = db::gds2_converter.to_char(db::sHEADER);
    // TODO: implement a more elaborate test for this format ...
    return hdr[0] == '#' || (hdr[0] == sHeader.at(0) && hdr[1] == sHeader.at(1) && hdr[2] == sHeader.at(2) && hdr[3] == sHeader.at(3));
  }

  virtual ReaderBase *create_reader (tl::InputStream &s) const 
  {
    return new db::GDS2ReaderText(s);
  }

  virtual WriterBase *create_writer () const 
  {
    return new db::GDS2WriterText();
  }
};

static tl::Registrar<db::StreamFormatDeclaration>::Class format_txt_decl (new GDS2TextFormatDeclaration(), 0, "GDS2Text");

//  provide a symbol to force linking against
int force_link_GDS2Text = 0;

}

