
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbLibraryManager.h"
#include "dbLibrary.h"
#include "tlClassRegistry.h"

#include <memory>

namespace db
{

template <> tl::Registrar<db::Library> *tl::Registrar<db::Library>::instance = 0;

static std::auto_ptr<LibraryManager> s_instance (0);

LibraryManager &
LibraryManager::instance ()
{
  if (s_instance.get () == 0) {
    s_instance.reset (new LibraryManager ());
  }
  return *s_instance.get ();
}

LibraryManager::LibraryManager ()
{
  // handle auto-registered libraries:
  for (tl::Registrar<db::Library>::iterator l = tl::Registrar<db::Library>::begin (); l != tl::Registrar<db::Library>::end (); ++l) {
    // Libraries must be non-const in order to allow PCell instantiation.
    register_lib (const_cast<Library *> (&*l));
  }
}

LibraryManager::~LibraryManager ()
{
  clear ();
}

std::pair<bool, lib_id_type> 
LibraryManager::lib_by_name (const std::string &name) const
{
  iterator l = m_lib_by_name.find (name);
  if (l == m_lib_by_name.end ()) {
    return std::make_pair (false, lib_id_type (0));
  } else {
    return std::make_pair (true, l->second);
  }
}

lib_id_type 
LibraryManager::register_lib (Library *library)
{
  lib_id_type id = m_libs.size ();
  m_libs.push_back (library);
  m_lib_by_name.insert (std::make_pair (library->get_name (), id)).first->second = id;
  library->set_id (id);
  return id;
}

Library *
LibraryManager::lib (lib_id_type id) const
{
  if (id >= m_libs.size ()) {
    return 0;
  } else {
    return m_libs [id];
  }
}

void
LibraryManager::clear ()
{
  for (std::vector<Library *>::iterator l = m_libs.begin (); l != m_libs.end (); ++l) {
    if (*l) {
      delete *l;
    }
  }
  m_libs.clear ();
  m_lib_by_name.clear ();
}

}

