
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbReader.h"
#include "dbStream.h"
#include "tlClassRegistry.h"

namespace db
{

// ---------------------------------------------------------------
//  Reader implementation

Reader::Reader (tl::InputStreamBase &stream_base)
  : mp_actual_reader (0), m_stream (stream_base)
{
  const char *hdr = m_stream.get (4);
  m_stream.unget (4);

  if (! hdr) {
    throw db::ReaderException (tl::translate ("Unable to determine stream format (too few bytes)"));
  }

  for (tl::Registrar<db::StreamFormatDeclaration>::iterator rdr = tl::Registrar<db::StreamFormatDeclaration>::begin (); rdr != tl::Registrar<db::StreamFormatDeclaration>::end () && ! mp_actual_reader; ++rdr) {
    if (rdr->detect (hdr)) {
      mp_actual_reader = rdr->create_reader (m_stream);
    }
  }
    
  if (! mp_actual_reader) {
    throw db::ReaderException (tl::translate ("Stream has unknown format"));
  }
}

Reader::~Reader ()
{
  if (mp_actual_reader) {
    delete mp_actual_reader;
    mp_actual_reader = 0;
  }
}

}

