
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_dbStatic
#define HDR_dbStatic

namespace db
{

// -----------------------------------------------------------
//  editable mode

extern bool ms_editable;

/**
 *  @brief Global database attribute: editable mode
 *
 *  This attribute reflects editable mode. In editable mode, some restrictions apply, i.e. 
 *  only shapes with attributes may be created.
 */
inline bool editable_mode ()
{
  return ms_editable;
}

/**
 *  @brief Set editable mode 
 *
 *  Hint: this should only be done initially, not at runtime.
 */
void set_editable_mode (bool editable);

/**
 *  @brief Check if in editable mode
 *
 *  This functions throws an exception if not in editable mode.
 *  "f_str" is reported as the function name.
 */
void check_editable_mode (const char *f_str);

/**
 *  @brief Check if in editable mode
 *
 *  This functions throws an exception if not in editable mode.
 */
void check_editable_mode ();

// -----------------------------------------------------------
//  transaction enable 

extern bool ms_transactions_enabled;

/**
 *  @brief Return true, if undo buffering is enabled
 */
inline bool transactions_enabled () 
{
  return ms_transactions_enabled;
}

/**
 *  @brief Enable or disable transaction buffering
 *
 *  @param enabled True to enable undo buffering
 */
void enable_transactions (bool enable);

}

#endif

