
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef _HDR_gsiClass
#define _HDR_gsiClass

//  For a comprehensive documentation see gsi.h

namespace gsi
{

template <class X> 
void *_take_and_create (SerialArgs &args, CantCopySemanticsTag)
{
  tl_assert (false); // cannot copy object of this type
  return 0;
}

template <class X> 
void *_take_and_create (SerialArgs &from, CanCopySemanticsTag)
{
  return from.template take_object<X> ();
}

template <class X> 
void *_get_vector_of (SerialArgs &from, bool ref, void *data, void (*cb) (void *data, void *obj), CantCopySemanticsTag) 
{
  tl_assert (false); // cannot copy object of this type
  return 0;
}

template <class X> 
void _get_vector_of (SerialArgs &from, bool ref, void *data, void (*cb) (void *data, void *obj), CanCopySemanticsTag) 
{
  std::vector<X> vv;
  const std::vector<X> *v = &vv;
  if (ref) {
    v = &from.template get_value<const std::vector<X> &> (typename gsi::type_traits<const std::vector<X> &>::tag ());
  } else {
    vv = from.template get_value< std::vector<X> > (typename gsi::type_traits< std::vector<X> >::tag ());
  }
  for (typename std::vector<X>::const_iterator o = v->begin (); o != v->end (); ++o) {
    (*cb) (data, new X (*o));
  }
}

template <class X> 
void _push_vector_of (SerialArgs &to, bool ref, std::list<TempObject> &temp_objects, const std::vector<void *> &objects, CantCopySemanticsTag) 
{
  tl_assert (false); // cannot copy object of this type
}

template <class X> 
void _push_vector_of (SerialArgs &to, bool ref, std::list<TempObject> &temp_objects, const std::vector<void *> &objects, CanCopySemanticsTag) 
{
  std::vector<X> vv;
  std::vector<X> *v;
  if (ref) {
    v = new std::vector<X> ();
    temp_objects.push_back (TempObject ());
    temp_objects.back ().set (v);
  } else {
    v = &vv;
  }

  for (std::vector<void *>::const_iterator o = objects.begin (); o != objects.end (); ++o) {
    v->push_back (*(X *)*o);
  }

  if (ref) {
    to.set_value (cref_tag (), *v);
  } else {
    to.set_value (direct_npod_tag (), vv);
  }
}

template <class X> 
void *_create (CantDefaultCreateSemanticsTag)
{
  throw tl::Exception (tl::translate ("Object cannot be created here"));
  return 0;
}

template <class X> 
void *_create (CanDefaultCreateSemanticsTag)
{
  return new X ();
}

template <class X> 
void _assign (CantCopySemanticsTag, void *, void *)
{
  throw tl::Exception (tl::translate ("Object cannot be copied here"));
}

template <class X> 
void _assign (CanCopySemanticsTag, void *dest, void *src)
{
  *(X *)dest = *(const X *)src;
}

template <class X> 
void _copy_to (SerialArgs &at, void *from, CantCopySemanticsTag) 
{
  throw tl::Exception (tl::translate ("Object cannot be copied here"));
}

template <class X> 
void _copy_to (SerialArgs &at, void *from, CanCopySemanticsTag) 
{
  at.set_value (typename type_traits<X>::tag (), *((const X *)from));
}

inline void *_get_client_data (void *) 
{
  return 0;
}

inline void *_get_client_data (gsi::ObjectBase *x) 
{
  return x->data ();
}

inline bool _can_copy (CanCopySemanticsTag)  { return true;  }
inline bool _can_copy (CantCopySemanticsTag) { return false; }

inline bool _can_default_create (CanDefaultCreateSemanticsTag)  { return true;  }
inline bool _can_default_create (CantDefaultCreateSemanticsTag) { return false; }


/**
 *  @brief The declaration of a specific class
 *
 *  This class declares all methods that are required to instantiate or copy an object
 *  or to call it's methods in some generic way.
 */
template <class X>
class Class
  : public ClassBase
{
public:
  typedef typename ClassTag<X>::copy_semantics copy_semantics_tag;
  typedef typename ClassTag<X>::create_semantics create_semantics_tag;

  Class (const std::string &name, const Methods &mm, const std::string &doc = std::string ())
    : ClassBase (name, 0, doc, mm)
  {
    // ...
  }

  Class (const ClassBase &base, const std::string &name, const Methods &mm, const std::string &doc = std::string ())
    : ClassBase (name, &base, doc, mm)
  {
    // ...
  }

  Class (const std::string &name, const std::string &doc = std::string ())
    : ClassBase (name, 0, doc, Methods ())
  {
    // ...
  }

  Class (const ClassBase &base, const std::string &name, const std::string &doc = std::string ())
    : ClassBase (name, &base, doc, Methods ())
  {
    // ...
  }

  void attach (Object<X> *obj) const
  {
    if (client_handler ()) {
      client_handler ()->attach (obj->mp_data);
    }
  }

  void detach (Object<X> *obj) const
  {
    if (client_handler () && obj->mp_data) {
      client_handler ()->detach (obj->mp_data);
    }
  }

  void *client_data (void *p) const
  {
    X *x = (X *)p;
    return _get_client_data (x);
  }

  virtual void destroy (void *p) const
  {
    X *x = (X *)p;
    delete x;
  }

  virtual void *create () const
  {
    create_semantics_tag cst;
    return _create<X> (cst);
  }

  virtual void assign (void *dest, void *src) const
  {
    copy_semantics_tag cst;
    _assign<X> (cst, dest, src);
  }

  virtual void get_vector_of (SerialArgs &from, bool ref, void *data, void (*cb) (void *data, void *obj)) const 
  {
    copy_semantics_tag cpt;
    _get_vector_of<X> (from, ref, data, cb, cpt);
  }

  void push_vector_of (SerialArgs &to, bool ref, std::list<TempObject> &temp_objects, const std::vector<void *> &objects) const 
  {
    copy_semantics_tag cpt;
    _push_vector_of<X> (to, ref, temp_objects, objects, cpt);
  }

  virtual void *take_and_create (SerialArgs &from) const
  {
    copy_semantics_tag cpt;
    return _take_and_create<X> (from, cpt);
  }

  virtual void copy_to (SerialArgs &at, void *from) const 
  {
    copy_semantics_tag cpt;
    _copy_to<X> (at, from, cpt);
  }

  virtual void *create_obj_iterator (SerialArgs &from) const
  {
    return from.template get_value<gsi::IterAdaptorAbstractBase *> (typename type_traits <gsi::IterAdaptorAbstractBase *>::tag ());
  }

  virtual bool obj_iterator_at_end (void *iter) const
  {
    gsi::IterAdaptorAbstractBase *i = (gsi::IterAdaptorAbstractBase *) iter;
    return i->at_end ();
  }

  virtual void *take_iterator_obj (void *iter) const
  {
    gsi::IterAdaptorAbstractBase *i = (gsi::IterAdaptorAbstractBase *) iter;
    return i->v_get_copy ();
  }

  virtual const void *get_iterator_obj (void *iter) const
  {
    gsi::IterAdaptorAbstractBase *i = (gsi::IterAdaptorAbstractBase *) iter;
    return i->v_get ();
  }

  virtual void inc_obj_iterator (void *iter) const
  {
    gsi::IterAdaptorAbstractBase *i = (gsi::IterAdaptorAbstractBase *) iter;
    i->inc ();
  }

  virtual void delete_obj_iterator (void *iter) const
  {
    gsi::IterAdaptorAbstractBase *i = (gsi::IterAdaptorAbstractBase *) iter;
    delete i;
  }

  virtual ClassBase *declaration () const
  {
    return &decl;
  }

  virtual bool can_copy () const
  {
    copy_semantics_tag cpt;
    return _can_copy (cpt);
  }

  virtual bool can_default_create () const
  {
    create_semantics_tag cpt;
    return _can_default_create (cpt);
  }

  static Class<X> decl;
};

}

#endif


