
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "dbCellMapping.h"
#include "dbLayout.h"

#include <memory>

namespace gsi
{

// static constructor for a cell mapping object
db::CellMapping *new_cell_mapping (const db::Layout &layout_a, db::cell_index_type cell_index_a, const db::Layout &layout_b, db::cell_index_type cell_index_b)
{
  std::auto_ptr<db::CellMapping> cell_mapping (new db::CellMapping ());
  cell_mapping->create (layout_a, cell_index_a, layout_b, cell_index_b);
  return cell_mapping.release ();
}

template <>
Class<db::CellMapping> Class<db::CellMapping>::decl ("CellMapping", 
  gsi::constructor ("new", &new_cell_mapping, 
    "@brief Create a new cell mapping\n"
    "\n"
    "@args layout_a, cell_index_a, layout_b, cell_index_b\n"
    "\n"
    "The cell mapping is created for cells below cell_a and cell_b in the respective layouts.\n"
  ) +
  gsi::method ("has_mapping?", &db::CellMapping::has_mapping, 
    "@brief Determine if a cell layout_b has a mapping to a layout_a cell.\n"
    "\n"
    "@args cell_index_b\n"
    "\n"
    "@param cell_index_b The index of the cell in layout_b whose mapping is requested.\n"
    "@return true, if the cell has a mapping\n"
  ) +
  gsi::method ("cell_mapping", &db::CellMapping::cell_mapping, 
    "@brief Determine cell mapping to a layout_b cell to the corresponding layout_a cell.\n"
    "\n"
    "@args cell_index_b\n"
    "\n"
    "@param cell_index_b The index of the cell in layout_b whose mapping is requested.\n"
    "@return the cell index in layout_a.\n"
  ),
  "@brief A cell mapping derived from two hierarchies\n"
  "\n"
  "A cell mapping is an association of cells in two layouts forming pairs of cells, i.e. "
  "on cell corresponds to another cell in the other layout. Correspondency is defined by "
  "exact identity of both flat instantiations in the given staring cell. Therefore, when "
  "a cell is mapped to another cell, shapes can be transferred from one cell to another "
  "while effectively rendering the same flat geometry (in the context of the given starting cells).\n"
  "\n"
  "A cell might not be mapped to another cell which basically means that there is no corresponding cell. "
  "In this case, flattening to the next mapped cell is an option to transfer geometries despite the "
  "missing mapping.\n"
  "\n"
  "A cell mapping is created by instantiating a cell mapping object. Pass two layouts and two "
  "starting cells to specify which cell trees to map.\n"
);

}

