
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layMainConfigPages.h"
#include "layConfig.h"
#include "layStream.h"
#include "layAbstractMenu.h"
#include "layMainWindow.h"
#include "MainConfigPage.h"
#include "MainConfigPage2.h"
#include "MainConfigPage3.h"
#include "KeyBindingsConfigPage.h"

#include <map>

// ------------------------------------------------------------
//  Declaration of the configuration options
//  The configuration pages are declared via two "dummy" plugins

namespace lay
{

const std::string cfg_grid ("grid-micron");
const std::string cfg_default_grids ("default_grids");
const std::string cfg_synchronized_views ("synchronized-views");
const std::string cfg_default_dbu ("dbu");
const std::string cfg_mru ("mru");
const std::string cfg_key_bindings ("key-bindings");
const std::string cfg_show_toolbar ("show-toolbar");
const std::string cfg_show_hierarchy_panel ("show-hierarchy-panel");
const std::string cfg_show_layer_panel ("show-layer-panel");

class MainPluginDeclaration
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_grid, "0.001"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Grid");
    return new MainConfigPage (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl (new MainPluginDeclaration (), 1000, "MainPlugin(1)");

class MainPluginDeclaration2
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_synchronized_views, "false"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Misc.");
    return new MainConfigPage2 (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl2 (new MainPluginDeclaration2 (), 1001, "MainPlugin(2)");

class MainPluginDeclaration3
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_default_grids, "0.01,0.005,0.001"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Grid");
    return new MainConfigPage3 (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl3 (new MainPluginDeclaration3 (), 1002, "MainPlugin(3)");

class MainPluginDeclaration4
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_default_dbu, "0.001"));
    options.push_back (std::pair<std::string, std::string> (cfg_mru, ""));
    options.push_back (std::pair<std::string, std::string> (cfg_reader_enable_text_objects, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_reader_enable_properties, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_show_toolbar, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_show_hierarchy_panel, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_show_layer_panel, "true"));
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl4 (new MainPluginDeclaration4 (), 1003, "MainPlugin(4)");

class KeyBindingsPluginDeclaration
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_key_bindings, ""));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Key Bindings");
    return new KeyBindingsConfigPage (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl5 (new KeyBindingsPluginDeclaration (), 1003, "KeyBindingsPlugin");

// -------------------------------------------------------------
//  The "grid" configuration page

MainConfigPage::MainConfigPage (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::MainConfigPage ();
  mp_ui->setupUi (this);
}

MainConfigPage::~MainConfigPage ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
MainConfigPage::setup (lay::Plugin *root)
{
  double grid_micron = 0.0;
  root->config_get (cfg_grid, grid_micron);
  mp_ui->grid_edit->setText (tl::to_string (grid_micron).c_str ());
}

void 
MainConfigPage::commit (lay::Plugin *root)
{
  try {
    double g;
    tl::from_string (mp_ui->grid_edit->text ().ascii (), g);
    root->config_set (cfg_grid, g);
  } catch (...) { }
}

// -------------------------------------------------------------
//  The "default grids" configuration page

MainConfigPage3::MainConfigPage3 (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::MainConfigPage3 ();
  mp_ui->setupUi (this);
}

MainConfigPage3::~MainConfigPage3 ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
MainConfigPage3::setup (lay::Plugin *root)
{
  std::string default_grids;
  root->config_get (cfg_default_grids, default_grids);
  mp_ui->grids_edit->setText (default_grids.c_str ());
}

void 
MainConfigPage3::commit (lay::Plugin *root)
{
  root->config_set (cfg_default_grids, mp_ui->grids_edit->text ().ascii ());
}

// ------------------------------------------------------------
//  The "misc" config page

MainConfigPage2::MainConfigPage2 (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::MainConfigPage2 ();
  mp_ui->setupUi (this);
}

MainConfigPage2::~MainConfigPage2 ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
MainConfigPage2::setup (lay::Plugin *root)
{
  bool flag = false;
  root->config_get (cfg_synchronized_views, flag);
  mp_ui->sync_views_cbx->setChecked (flag);
}

void 
MainConfigPage2::commit (lay::Plugin *root)
{
  root->config_set (cfg_synchronized_views, mp_ui->sync_views_cbx->isChecked ());
}

// ------------------------------------------------------------
//  The "key bindings" config page

std::vector<std::pair<std::string, std::string> > 
unpack_key_binding (const std::string &packed)
{
  tl::Extractor ex (packed.c_str ());

  std::vector<std::pair<std::string, std::string> > key_bindings;

  while (! ex.at_end ()) {
    ex.test(";");
    key_bindings.push_back (std::make_pair (std::string (), std::string ()));
    ex.read_word_or_quoted (key_bindings.back ().first);
    ex.test(":");
    ex.read_word_or_quoted (key_bindings.back ().second);
  }

  return key_bindings;
}

std::string 
pack_key_binding (const std::vector<std::pair<std::string, std::string> > &unpacked)
{
  std::string packed;

  for (std::vector<std::pair<std::string, std::string> >::const_iterator p = unpacked.begin (); p != unpacked.end (); ++p) {
    if (! packed.empty ()) {
      packed += ";";
    }
    packed += tl::to_word_or_quoted_string (p->first);
    packed += ":";
    packed += tl::to_word_or_quoted_string (p->second);
  }

  return packed;
}

KeyBindingsConfigPage::KeyBindingsConfigPage (QWidget *parent)
  : lay::ConfigPage (parent), m_enable_event (true)
{
  mp_ui = new Ui::KeyBindingsConfigPage ();
  mp_ui->setupUi (this);
  connect (mp_ui->bindings_list, SIGNAL (currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), this, SLOT (current_changed (QTreeWidgetItem *, QTreeWidgetItem *)));
}

KeyBindingsConfigPage::~KeyBindingsConfigPage ()
{
  delete mp_ui;
  mp_ui = 0;
}

static void fill_paths (const lay::AbstractMenu &menu, const std::string &root, std::map<std::string, std::string> &bindings)
{
  std::vector<std::string> items = menu.items (root);
  for (std::vector<std::string>::const_iterator i = items.begin (); i != items.end (); ++i) {
    //  only choose items from the top-level menu (path not starting with a '@')
    if (i->size () > 0 && (*i)[0] != '@') {
      if (menu.is_valid (*i) && menu.action (*i).is_visible ()) {
        if (menu.is_menu (*i)) {
          fill_paths (menu, *i, bindings);
        } else if (! menu.is_separator (*i)) {
          bindings.insert (std::make_pair (*i, menu.action (*i).get_shortcut ()));
        }
      }
    }
  }
}

void 
KeyBindingsConfigPage::setup (lay::Plugin *root)
{
  m_enable_event = false;

  std::string packed_key_bindings;
  root->config_get (cfg_key_bindings, packed_key_bindings);
  std::vector<std::pair<std::string, std::string> > key_bindings = unpack_key_binding (packed_key_bindings);

  //  get the current bindings
  m_current_bindings.clear ();
  fill_paths (lay::MainWindow::instance ().menu (), std::string (), m_current_bindings);

  //  overwrite with the given ones
  for (std::vector<std::pair<std::string, std::string> >::const_iterator kb = key_bindings.begin (); kb != key_bindings.end (); ++kb) {
    std::map<std::string, std::string>::iterator cb = m_current_bindings.find (kb->first);
    if (cb != m_current_bindings.end ()) {
      cb->second = kb->second;
    }
  }

  //  fill the bindings list
  mp_ui->bindings_list->clear ();
  for (std::map<std::string, std::string>::const_iterator cb = m_current_bindings.begin (); cb != m_current_bindings.end (); ++cb) {
    QTreeWidgetItem *item = new QTreeWidgetItem (mp_ui->bindings_list);
    item->setData (0, Qt::DisplayRole, QString (cb->first.c_str ()));
    item->setData (1, Qt::DisplayRole, QString (lay::MainWindow::instance ().menu ().action (cb->first).get_title ().c_str ()));
    item->setData (2, Qt::DisplayRole, QString (cb->second.c_str ()));
  }

  m_enable_event = true;
}

void 
KeyBindingsConfigPage::commit (lay::Plugin *root)
{
  current_changed (0, mp_ui->bindings_list->currentItem ());

  //  Because the available key bindings change in edit and viewer mode, we always extend the key bindings but never 
  //  reduce them.

  std::string packed_key_bindings;
  root->config_get (cfg_key_bindings, packed_key_bindings);
  std::vector<std::pair<std::string, std::string> > key_bindings = unpack_key_binding (packed_key_bindings);

  for (std::vector<std::pair<std::string, std::string> >::iterator kb = key_bindings.begin (); kb != key_bindings.end (); ++kb) {
    std::map<std::string, std::string>::iterator cb = m_current_bindings.find (kb->first);
    if (cb != m_current_bindings.end ()) {
      kb->second = cb->second;
      m_current_bindings.erase (cb);
    }
  }

  for (std::map<std::string, std::string>::const_iterator cb = m_current_bindings.begin (); cb != m_current_bindings.end (); ++cb) {
    key_bindings.push_back (*cb);
  }

  packed_key_bindings = pack_key_binding (key_bindings);
  root->config_set (cfg_key_bindings, packed_key_bindings);
}

void 
KeyBindingsConfigPage::current_changed (QTreeWidgetItem *current, QTreeWidgetItem *previous)
{
  if (! m_enable_event) {
    return;
  }

  if (previous) {

    QKeySequence key_sequence (mp_ui->binding_le->text ());
    previous->setData (2, Qt::DisplayRole, key_sequence.toString ());

    std::string path = previous->data (0, Qt::DisplayRole).toString ().ascii ();
    std::string shortcut = previous->data (2, Qt::DisplayRole).toString ().ascii ();

    m_current_bindings[path] = shortcut;

  }

  if (current) {
    mp_ui->binding_le->setText (current->data (2, Qt::DisplayRole).toString ());
  } else {
    mp_ui->binding_le->setText ("");
  }
}

}

