
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layStream.h"
#include "layPlugin.h"
#include "layConfig.h"
#include "dbStream.h"
#include "dbLoadLayoutOptions.h"
#include "dbSaveLayoutOptions.h"
#include "tlAssert.h"

namespace lay
{

// ------------------------------------------------------------------
//  StreamPluginDeclarationBase implementation

db::StreamFormatDeclaration &
StreamPluginDeclarationBase::stream_fmt () 
{
  if (! mp_stream_fmt) {
    for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end (); ++fmt) {
      if (fmt->format_name () == m_format_name) {
        mp_stream_fmt = const_cast <db::StreamFormatDeclaration *> (&*fmt);
        break;
      }
    }
  }

  tl_assert (mp_stream_fmt);

  return *mp_stream_fmt;
}

void commit_to_config (const db::LoadLayoutOptions &options, lay::PluginRoot *config_root)
{
  //  Store all format specific options to the configuration
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
    const StreamReaderPluginDeclaration *decl = dynamic_cast <const StreamReaderPluginDeclaration *> (&*cls);
    if (decl) {
      const db::FormatSpecificReaderOptions *specific_options = options.get_specific_options (decl->format_name ());
      if (specific_options) {
        decl->commit_to_config (specific_options, config_root);
      }
    }
  }

  //  Establish persistency for the persistent properties that MainWindow is responsible for.
  //  Hint: this will again set the parameters to the same values, but that is not easy to avoid ...
  config_root->config_set (cfg_reader_enable_text_objects, options.enable_text_objects ());
  config_root->config_set (cfg_reader_enable_properties, options.enable_properties ());

  config_root->config_end ();
}

void commit_to_config (const db::SaveLayoutOptions &options, lay::PluginRoot *config_root)
{
  //  Store all format specific options to the configuration
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
    const StreamWriterPluginDeclaration *decl = dynamic_cast <const StreamWriterPluginDeclaration *> (&*cls);
    if (decl && decl->format_name () == options.format ()) {
      const db::FormatSpecificWriterOptions *specific_options = options.get_specific_options ();
      if (specific_options) {
        decl->commit_to_config (specific_options, config_root);
      }
      break;
    }
  }

  //  Establish persistency for the persistent properties that MainWindow is responsible for.
  //  TODO: implement, once there are some defined ...

  config_root->config_end ();
}

// ------------------------------------------------------------------
//  A "basic" reader plugin to declare the global reader configuration options

const std::string cfg_reader_enable_text_objects ("reader-enable-text-objects");
const std::string cfg_reader_enable_properties ("reader-enable-properties");

class BasicStreamReaderPlugin
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_reader_enable_text_objects, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_reader_enable_properties, "true"));
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl_reader (new BasicStreamReaderPlugin(), 1100, "BasicStreamReader");


}

